@@ -112,7 +112,8 @@ class Accessory(object):
         mac = util.generate_mac()
         return cls(display_name, aid=aid, mac=mac, pincode=pincode)
 
-    def __init__(self, display_name, aid=None, mac=None, pincode=None, iid_manager=None):
+    def __init__(self, display_name, aid=STANDALONE_AID, mac=None, pincode=None,
+                 iid_manager=None):
 
         self.display_name = display_name
         self.aid = aid
@@ -145,7 +146,6 @@ class Accessory(object):
         The default implementation adds only the AccessoryInformation services
         and sets its Name characteristic to the Accessory's display name.
         """
-        # Info service
         info_service = get_serv_loader().get("AccessoryInformation")
         info_service.get_characteristic("Name")\
                     .set_value(self.display_name, False)
@@ -171,6 +171,22 @@ class Accessory(object):
         """
         self.run_sentinel = run_sentinel
 
+    def config_changed(self):
+        """Notify the accessory about configuration changes.
+
+        These include new services or updated characteristic values, e.g.
+        the Name of a service changed.
+
+        This method also notifies the broker about the change, so that it can
+        publish the changes to the world.
+
+        @note: If you are changing the configuration of a bridged accessory
+        (i.e. an Accessory that is contained in a Bridge),
+        you should call the `config_changed` method on the Bridge.
+        """
+        self.config_version += 1
+        self.broker.config_changed()
+
     def add_service(self, *servs):
         """Add the given services to this Accessory.
 
@@ -298,8 +314,13 @@ class Bridge(Accessory):
 
     category = Category.BRIDGE
 
-    def __init__(self, display_name, **kwargs):
-        super(Bridge, self).__init__(display_name, aid=STANDALONE_AID, **kwargs)
+    def __init__(self, display_name, mac=None, pincode=None, iid_manager=None):
+        aid = STANDALONE_AID
+        # A Bridge cannot be Bridge, hence talks directly to HAP clients.
+        # Thus, we need a mac.
+        mac = mac or util.generate_mac()
+        super(Bridge, self).__init__(display_name, aid=aid, mac=mac,
+                                     pincode=pincode, iid_manager=iid_manager)
         self.accessories = {}  # aid: acc
 
     def _set_services(self):
